/*
 * user.cpp
 *
 *  Created on: 27 мая 2015 г.
 *      Author: nick
 */


#include <user.h>
#include <httpd.h>
#include <vector.h>

static int32_t ADCReadTemp();

typedef struct _Temp {
	uint32_t time;
	int32_t degrees;
	_Temp() {
		degrees =  ADCReadTemp();
		time =  System->getTicksCount() / 1000;
	}
} Temp;

class MyHTTPD : public HTTPD {
private:
private:
  Vector<Temp *> _temps;
  uint64_t _lastMeasureTime;
protected:
	/**
	 * Обработка входящих запросов
	 */
	virtual void handleRequest(Socket *socket,char *path);
	/**
	 * Обработка подстановки переменных
	 */
	virtual void handleVariable(Socket *socket,const char *var);
public:
	MyHTTPD() : HTTPD(80,25000),_temps(Vector<Temp *>::defaultDestructor),_lastMeasureTime(0) { }
	/**
	 * 	Метод, вызываемый в режиме ожидания супердемона
	 */
	virtual void onIdle();
};

/*********************** Работа с ADC ******************************************/
static int16_t tempCal;

static uint8_t ADCReadCalibrationByte(uint8_t index) {
	uint8_t result;
	NVM_CMD = NVM_CMD_READ_CALIB_ROW_gc;
	result = pgm_read_byte(index);
	NVM_CMD = NVM_CMD_NO_OPERATION_gc;
	return result;
}

static void ADCInit() {
	ADCA.CALL = ADCReadCalibrationByte( offsetof(NVM_PROD_SIGNATURES_t, ADCACAL0) );
	ADCA.CALH = ADCReadCalibrationByte( offsetof(NVM_PROD_SIGNATURES_t, ADCACAL1) );

	ADCA.CTRLB = ADC_RESOLUTION_12BIT_gc;
	ADCA.REFCTRL = ADC_REFSEL_INT1V_gc | ADC_TEMPREF_bm | ADC_BANDGAP_bm ;
	ADCA.PRESCALER = ADC_PRESCALER_DIV256_gc;             // (32MHz/256=125kHz)


	ADCA.CH0.CTRL = ADC_CH_INPUTMODE_INTERNAL_gc;
	ADCA.CH0.MUXCTRL = ADC_CH_MUXINT_TEMP_gc;

	ADCA.CTRLA |= ADC_ENABLE_bm;
	tempCal = ADCReadCalibrationByte( offsetof(NVM_PROD_SIGNATURES_t, TEMPSENSE0) );
	tempCal |= (ADCReadCalibrationByte( offsetof(NVM_PROD_SIGNATURES_t, TEMPSENSE1)) <<8);

}


static inline int32_t ADCOffset(int32_t data) {
	int32_t m_offset;
	m_offset = (data-(0.05*1.08*4096));
	return m_offset;
}

static int32_t ADCReadTemp() {
	ADCA.CH0.CTRL |= ADC_CH_START_bm;
	while(ADCA.CH0.INTFLAGS != ADC_CH_CHIF_bm);
	ADCA.CH0.INTFLAGS = ADC_CH_CHIF_bm;
	int16_t meas = ADCA.CH0.RES;
	int32_t m_temp;
	int32_t m_omeas;
	int32_t m_ocal;
	m_omeas = ADCOffset(meas);
	m_ocal = ADCOffset(tempCal);
	m_temp = (m_omeas * (85+273))/(m_ocal) - 273;
	if(m_temp >= 70)
			DACB.CH0DATAH = 0;
	else if(m_temp <= 60)
			DACB.CH0DATAH = 10;
	return m_temp;
}
/************************************** Методы MyHTTPD *******************************/
void MyHTTPD::onIdle() {
	if(System->getTicksCount() - _lastMeasureTime >= 1000) {
		_lastMeasureTime = System->getTicksCount();
		if(_temps.size() == 10) // Значений 10, удаляем первое.
			_temps >> _temps.peek();
		_temps << new Temp(); // Создаем и добавляем новое
	}
}

void MyHTTPD::handleRequest(Socket *socket,char *path) {
	if(strcmp(path,"/tempGrid.json") == 0) {
		sendHeader(socket,s200,json); // Отправляем заколовок с HTTP статусом 200 OK и типом данных json
		char buffer[128];
		*socket << '[';
		for(Vector<Temp *>::Iterator i = _temps.begin();i!=_temps.end();i++) {
			if(i != _temps.begin()) // Элементы в массиве разделены ,
				*socket << ',';
			sprintf(buffer,"{\"time\":%lu,\"temp\":%ld}",(*i)->time,(*i)->degrees); // Формируем объект
			*socket << buffer; // Отправляем его
		}
		*socket << ']';
	}
	else // Иначе вызываем обработчик предка
		HTTPD::handleRequest(socket,path);
}

void MyHTTPD::handleVariable(Socket *socket,const char *var) {
	if(strcmp(var,"tempGrid") == 0) { // если имя переменной tempGrid
		char buffer[128];
		for(Vector<Temp *>::Iterator i = _temps.begin();i!=_temps.end();i++) { // Проходимся по всем элементам вектора
			sprintf(buffer,"<tr><td>%lu</td><td>%ld</td></tr>",(*i)->time,(*i)->degrees); // Формируем строку таблицы
			*socket << buffer; // И отправляем ее в соединение
		}
	} else // Иначе вызываем метод предка
		HTTPD::handleVariable(socket,var);
}
/******************************* Функции инициализации *********************************/
void setup(const bool reset) {
	ADCInit(); // Инициализируем ADC
	new VFS(VFS_USE_MAX_SIZE); // Инициализация файловой системы на Flash RAM, выделяем максимальный размер
	MyHTTPD *httpd = new MyHTTPD(); // Создаем экземпляр MyHTTPD
	SocketServer->registerSocketListener(httpd); // Регистрируем его в супердемоне
	// Задаем сетевые адреса по умолчанию
	IP4Address address = { 192,168,0,100};
	IP4Address mask = { 255,255,255,0};
	// Конфигурируем супердемон с использованием DHCP
	SocketServer->configure(true,address,mask);
}

void onIdle() { }

bool acceptConnection(const EthernetAddress hwAddr, const IP4Address ipAddr) {
	return true;
}
